/* A very simple A/UX example hybrid application, which polls the user's UNIX
   mailbox and uses the Notification Manager to inform the user when mail
   has arrived. It determines when the mail has been read by checking the 
   access date of the mailbox file. This should work with most UNIX mailers 
   but your mileage may vary.
   
   Paul Davison, UK Macintosh Developer Technical Support
   
   2nd September 1992
   
   - 10th November 1992 Notify if the mailbox is non-empty on startup.
						
 */


#include <Types.h>
#include <Notification.h>
#include <Events.h>
#include <Windows.h>
#include <Resources.h>
#include <Menus.h>
#include <ToolUtils.h>
#include <Dialogs.h>
#include <Desk.h>
#include <OSEvents.h>
#include <StdIO.h>
#include <String.h>
#include <Strings.h>

#include <LibAUX.h>							/* A/UX specific includes here */
#include </:usr:include:sys:types.h>
#include </:usr:include:sys:stat.h>

#define SLEEP 	300L						/* change these as appropriate */
#define TPERIOD 30

#define mApple 	128
#define mFile  	129
#define rAboutAlert 128
#define iQuit 	1
#define iAbout 	1

NMRec mystruct, saystruct;					/* lots of global variables but it's */
Str255 p;									/* only a sample....				 */
char *mbox;

long mboxSize, mboxADate;

int ProcessMenu();
int CheckMailBox();
int notifying = 0;

mySay( char *);

Boolean gInBackground;

Terminate() 
{
		if (notifying) NMRemove (&mystruct);
			/* if the notification has been done, "notifying" will be non-zero */
		ExitToShell();
}

sleep()
{
        EventRecord theEvent;
		unsigned long theTime, theStartTime;
		WindowPtr window;
		short part;
		long choice;
		
								/* this stuff modelled on TESample */
		while (1) {
			GetDateTime (&theStartTime);
			do {
					if (WaitNextEvent(everyEvent, &theEvent, SLEEP, 0L)) {
						
						switch(theEvent.what) {
							case mouseDown:
								part = FindWindow(theEvent.where, &window);
								if (part == inMenuBar) {
									choice = MenuSelect(theEvent.where);
									ProcessMenu(choice);
								}
								break;
							case keyDown:
							case autoKey:
								if (theEvent.modifiers & cmdKey) 
									ProcessMenu(MenuKey(theEvent.message & charCodeMask));
								break;
							case nullEvent: 
							default: break;
						}
					}
					GetDateTime(&theTime);
			} while (theTime < theStartTime + TPERIOD);
			CheckMailBox();
		}
}
	
ProcessMenu(what)
long what;
{
		short menuID, menuItem;
		Str255 daName;
		
		menuID = HiWord(what);
		menuItem = LoWord(what);
		
		switch (menuID) {
			case mApple:
				switch(menuItem) {
					case iAbout: Alert(rAboutAlert, nil);
								 break;
					default:	 GetItem(GetMHandle(mApple),menuItem,daName); 
							     OpenDeskAcc(daName);
								 break;
				}
				break;
			case mFile:
				switch(menuItem) {
					case iQuit:	 Terminate();
								 break;
				}
				break;
		}
}
				
doMenus()
{
		MenuHandle AppleMenu, FileMenu;
		
		AppleMenu = GetMenu(mApple);
		AddResMenu(AppleMenu, (ResType) 'DRVR');
		FileMenu = GetMenu(mFile);
		InsertMenu(AppleMenu,0);
		InsertMenu(FileMenu,0);
		DrawMenuBar();
}

SetUpNotifier()
{
		Handle mysound, myicon;
		
		/* Here we basically set up the notification structure ready for use */
		mysound = GetResource('snd ', 666);
		myicon = GetResource('SICN', 128);
        mystruct.qType = nmType;
        mystruct.nmMark = 0;
        mystruct.nmIcon = myicon;
        mystruct.nmSound =  mysound ;
        mystruct.nmStr = nil;
        mystruct.nmResp = (NMProcPtr) 0;
        mystruct.nmRefCon = nil;
}

InitMboxData()
{		
		int result;
		struct stat mystat;
		
		mbox = auxgetenv("MAIL");		/* use the MAIL environment variable to 
										   work out the mailbox pathname */
		if (mbox == NULL) {
			mySay("\pI don't know where your mailbox lives!");
			Terminate();
		}
		result = auxstat(mbox, &mystat);	/* do the initial stat to get file times */
		if (result == 0) {
			mboxSize = mystat.st_size;
			mboxADate = mystat.st_atime;
			if(mboxSize) {
				/* We have stuff in the mailbox - let's notify now. */
				NMInstall(&mystruct);
				notifying = 20;
			}
		}
}

CheckMailBox()
{
		int result;
		struct stat mystat;

		
		result = auxstat(mbox, &mystat);
		if (result < 0) {
			mboxSize = mboxADate = 0;
			if (notifying) {
				NMRemove(&mystruct);
				notifying=0;
			}
			return;
		}
		if (mystat.st_size > mboxSize && mboxADate <= mystat.st_atime) {	
				/* new stuff has arrived and hasn't been read already */
				
				if (!notifying) {
					NMInstall(&mystruct);
					notifying = 20;	
				}
				mboxSize = mystat.st_size;
				mboxADate = mystat.st_atime;
				return;
		}
		if (mystat.st_size < mboxSize) {	/* it shrunk - just catch up! */
				mboxSize = mystat.st_size;
				mboxADate = mystat.st_atime;
				if (notifying) {
					NMRemove(&mystruct);
					notifying=0;
				}
				return;
		}
		if (notifying && (mystat.st_atime > mboxADate)) { /* it's been read */
				NMRemove (&mystruct);
				notifying = 0;
				mboxSize = mystat.st_size;
				mboxADate = mystat.st_atime;
				return;
		} else if (notifying) {
				if(--notifying == 0) {		/* time for another notification? */
					NMRemove (&mystruct);
					NMInstall(&mystruct); /* do it again! */
					notifying = 20;
				}
		}
}

mySay(mess)
char *mess;
{
		strcpy(p,mess);	
        saystruct.qType = nmType;
        saystruct.nmMark = 0;
        saystruct.nmIcon = (Handle) nil;
        saystruct.nmSound =  (Handle) -1 ;
        saystruct.nmStr = p;
        saystruct.nmResp = (NMProcPtr) -1;
        saystruct.nmRefCon = nil;
		NMInstall(&saystruct);
}


main()
{      
		InitGraf(&qd.thePort);
		InitWindows();
		InitMenus();
		InitDialogs(0);
		InitCursor();
		doMenus();
		SetUpNotifier();
		
		if (!AUXisRunning()) {
				mySay("\pSorry, this application only works under A/UX!");
				return(0);
		}
		
		InitMboxData();
		sleep();
}
